const fs = require('fs')
const { resolve, join } = require('path')
const child_process = require('child_process')
const shell = require('shelljs')
const { useOra } = require('./hooks')
const { prompt } = require('inquirer')
const spawn = require('cross-spawn');
const { reactConfig } = require('./constant')

module.exports = async () => {
  /* 
    1. 拷贝配置文件
    2. 追加依赖
    3. 安装依赖
  */
  // 读取当前环境下package.json
  /* const json = fs.readFileSync(resolve(process.cwd(), 'package.json'), 'utf-8')
  console.log(JSON.parse(json)); */
  // 追加各种依赖和配置
  console.log('拷贝安装依赖配置');
  copyDep()
  console.log('添加依赖文件');
  const confirm = await prompt([
    {
      type: 'confirm',
      message: '是否需要选择一套框架',
      name: 'isSelect'
    }
  ])
  let depName
  if (confirm.isSelect) {
    const chooseRes = await prompt([
      {
        type: 'list',
        choices: [{ name: 'React', value: reactConfig }, { name: 'Vue', value: 'vue' }],
        name: 'depName',
        message: '选择一套框架'
      }
    ])
    depName = chooseRes.depName
  }
  appendDep(depName)
  console.log('初始化commit-lint');
  initProject(confirm.isSelect)
}

function copyDep() {
  const _temp = resolve(__dirname, '../template')
  const files = fs.readdirSync(_temp)
  files.filter(item => item !== 'package.json' && item !== 'package-temp.json').forEach(file => {
    const filePath = join(_temp, file)
    copy(filePath, resolve(process.cwd(), `./${file}`))
  })
}

function copy(orgUrl, destUrl) {
  const stat = fs.statSync(orgUrl)
  if (stat.isDirectory()) {
    if (fs.existsSync(destUrl)) {
      fs.rmSync(destUrl, { recursive: true })
    }
    fs.mkdirSync(join(destUrl))
    copyDir(orgUrl, destUrl)
  } else {
    fs.copyFileSync(orgUrl, destUrl)
  }
}

function copyDir(orgUrl, destUrl) {
  const files = fs.readdirSync(orgUrl)
  files.forEach(file => {
    const stat = fs.statSync(join(orgUrl, file))
    if (stat.isDirectory()) {
      copy(
        join(orgUrl, file),
        join(destUrl, file)
      )
    } else {
      fs.copyFileSync(
        join(orgUrl, file),
        join(destUrl, file)
      )
    }
  })
}

function appendDep(depName = {}) {
  const orgPkg = resolve(process.cwd(), 'package.json')
  const tempPkg = resolve(__dirname, '../template/package-temp.json')
  const orgPkgExist = fs.existsSync(orgPkg)
  if (!orgPkgExist) { // 不存在 直接拷贝
    return fs.copyFileSync(tempPkg, orgPkg)
  }
  let tplPkgObj = fs.readFileSync(tempPkg, { 'encoding': 'utf-8' }),
      orgPkgObj = fs.readFileSync(orgPkg, { 'encoding': 'utf-8' })
  tplPkgObj = JSON.parse(tplPkgObj)
  orgPkgObj = JSON.parse(orgPkgObj)
  const { devDependencies, config, scripts } = tplPkgObj
  Object.assign(orgPkgObj.devDependencies, devDependencies, depName)
  Object.assign(orgPkgObj.config, config)
  Object.assign(orgPkgObj.scripts, scripts)
  fs.writeFileSync(orgPkg, JSON.stringify(orgPkgObj, null, 2))
}

async function initProject(hasChoose) {
  if (!fs.existsSync(resolve(process.cwd(), '.git'))) {
    child_process.execSync('git init', {
      'cwd': resolve(process.cwd())
    })
  }
  const std = child_process.spawn('yarn.cmd', {
    'cwd': resolve(process.cwd()),
    stdio: 'inherit'
  })
  const { load, succeed } = useOra()
  console.log(process.env.npm_config_user_agent);
  load('开始安装');
  std.on('close', async () => {
    succeed('安装结束');
    if (!hasChoose) {
      const res = await prompt([
        {
          type: 'confirm',
          message: '已准备就绪，是否立刻初始化eslint',
          name: 'init',
        }
      ])
      if (res.init) {
        eslintInit()
      }
    }
  })
}

function eslintInit() {
  spawn('yarn', ['lintInit'], {
    cwd: resolve(process.cwd()),
    stdio: 'inherit'
  })
}